@@ -30,7 +30,11 @@ import apt_pkg
 import tempfile
 from gettext import gettext as _
 import os
-import subprocess
+
+import gi
+gi.require_version('PackageKitGlib', '1.0')
+from gi.repository import PackageKitGlib as packagekit
+from gi.repository import Gio
 
 from PyQt5 import uic
 from PyQt5.QtCore import *
@@ -53,8 +57,6 @@ from UbuntuDrivers import detect
 import sys
 import apt
 from functools import partial
-from aptdaemon import client
-from aptdaemon.errors import NotAuthorizedError, TransactionFailed
 import aptsources.distro
 import logging
 
@@ -155,8 +157,6 @@ class SoftwarePropertiesQt(SoftwareProperties):
     self.init_distro()
     self.show_distro()
 
-    self.apt_client = client.AptClient()
-
     if options and options.open_tab:
       self.userinterface.tabWidget.setCurrentIndex(int(options.open_tab))
 
@@ -739,6 +739,20 @@ class SoftwarePropertiesQt(SoftwareProperties):
     self.apt_key.update()
     self.show_keys()
 
+  def on_pktask_progress(self, progress, ptype, udata=(None,)):
+    if ptype == packagekit.ProgressType.PERCENTAGE:
+      perc = progress.get_property('percentage')
+      self._pdialog.setValue(perc)
+
+  def on_pktask_finish(self, source, result, udata=(None,)):
+    results = None
+    try:
+        results = self._pktask.generic_finish(result)
+    except Exception as e:
+      QMessageBox.warning(self.userinterface, _("Could not refresh cache"), str(e))
+    self._pdialog.hide()
+    kapp.quit()
+
   def on_close_button(self):
     """Show a dialog that a reload of the channel information is required
        only if there is no parent defined"""
@@ -753,9 +767,34 @@ class SoftwarePropertiesQt(SoftwareProperties):
         messageBox.setText(text)
         messageBox.exec_()
         if (messageBox.clickedButton() == reloadButton):
-                cmd = ["/usr/bin/qapt-batch", "--update"]
-                subprocess.call(cmd)
-    kapp.quit()
+                self._pdialog = QProgressDialog("<big>{}</big>".format(_("Refreshing software cache")),
+                                                "Cancel", 0, 100, self.userinterface)
+                self._pdialog.setWindowTitle(_("Cache Refresh"))
+                self._pdialog.setCancelButton(None)
+                self._pdialog.setAutoClose(False)
+                self._pdialog.setMinimumWidth(210)
+                self._pdialog.setMinimumHeight(60)
+
+                self._pktask = packagekit.Task()
+                self._pdialog.show()
+                self.userinterface.hide()
+                try:
+                    self._pktask.refresh_cache_async (False, # force
+                                                  None,  # GCancellable
+                                                  self.on_pktask_progress,
+                                                  (None,), # user data
+                                                  self.on_pktask_finish,
+                                                  (None,));
+                except Exception as e:
+                    print("Error while requesting cache refresh: {}".format (e))
+        else:
+            # refresh not wanted, quit immediately
+            kapp.quit()
+    else:
+        # no changes, no cache refresh needed.
+        # we can quit.
+        kapp.quit()
+
 
   def on_button_add_cdrom_clicked(self):
     '''Show a dialog that allows to add a repository located on a CDROM
@@ -797,7 +836,7 @@ class SoftwarePropertiesQt(SoftwareProperties):
   def run(self):
     kapp.exec_()
 
-  def on_driver_changes_progress(self, transaction, progress):
+  def on_driver_changes_progress(self, progress, ptype, data=None):
     self.button_driver_revert.setVisible(False)
     self.button_driver_apply.setVisible(False)
     self.button_driver_restart.setVisible(False)
@@ -805,17 +844,27 @@ class SoftwarePropertiesQt(SoftwareProperties):
     self.progress_bar.setVisible(True)
 
     self.userinterface.label_driver_action.setText("Applying changes...")
-    self.progress_bar.setValue(progress)
+    if ptype == packagekit.ProgressType.PERCENTAGE:
+        prog_value = progress.get_property('percentage')
+        self.progress_bar.setValue(prog_value)
 
-  def on_driver_changes_finish(self, transaction, exit_state):
-    self.progress_bar.setVisible(False)
-    self.clear_changes()
-    self.apt_cache = apt.Cache()
-    self.set_driver_action_status()
-    self.update_label_and_icons_from_status()
-    self.button_driver_revert.setVisible(True)
-    self.button_driver_apply.setVisible(True)
-    self.button_driver_cancel.setVisible(False)
+  def on_driver_changes_finish(self, source, result, installs_pending):
+    results = None
+    try:
+      results = self.pk_task.generic_finish(result)
+    except Exception as e:
+      self.on_driver_changes_revert()
+      QMessageBox.warning(self.userinterface, _("Error while applying changes"), str(e))
+
+    if not installs_pending:
+      self.progress_bar.setVisible(False)
+      self.clear_changes()
+      self.apt_cache = apt.Cache()
+      self.set_driver_action_status()
+      self.update_label_and_icons_from_status()
+      self.button_driver_revert.setVisible(True)
+      self.button_driver_apply.setVisible(True)
+      self.button_driver_cancel.setVisible(False)
     #self.scrolled_window_drivers.set_sensitive(True)
 
   def on_driver_changes_error(self, transaction, error_code, error_details):
@@ -831,13 +880,14 @@ class SoftwarePropertiesQt(SoftwareProperties):
     self.button_driver_cancel.setEnabled(cancellable)
 
   def on_driver_changes_apply(self, button):
+    self.pk_task = packagekit.Task()
     button = self.userinterface.sender()
     installs = []
     removals = []
 
     for pkg in self.driver_changes:
       if pkg.is_installed:
-          removals.append(pkg.shortname)
+          removals.append(self.get_package_id(pkg.installed))
           # The main NVIDIA package is only a metapackage.
           # We need to collect its dependencies, so that
           # we can uninstall the driver properly.
@@ -845,22 +895,37 @@ class SoftwarePropertiesQt(SoftwareProperties):
             for dep in get_dependencies(self.apt_cache, pkg.shortname, 'nvidia'):
               dep_pkg = self.apt_cache[dep]
               if dep_pkg.is_installed:
-                removals.append(dep_pkg.shortname)
+                removals.append(self.get_package_id(dep_pkg.installed))
       else:
-        installs.append(pkg.shortname)
+        installs.append(self.get_package_id(pkg.candidate))
 
+    self.cancellable = Gio.Cancellable()
     try:
-      self.transaction = self.apt_client.commit_packages(install=installs, remove=removals, reinstall=[], purge=[], upgrade=[], downgrade=[])
-      self.transaction.connect('progress-changed', self.on_driver_changes_progress)
-      self.transaction.connect('cancellable-changed', self.on_driver_changes_cancellable_changed)
-      self.transaction.connect('finished', self.on_driver_changes_finish)
-      self.transaction.connect('error', self.on_driver_changes_error)
-      self.transaction.set_debconf_frontend("kde")
-      self.transaction.run()
+      if removals:
+        installs_pending = False
+        if installs:
+            installs_pending = True
+        self.pk_task.remove_packages_async(removals,
+                  False,  # allow deps
+                  True,  # autoremove
+                  self.cancellable,  # cancellable
+                  self.on_driver_changes_progress,
+                  (None, ),  # progress data
+                  self.on_driver_changes_finish,  # callback ready
+                  installs_pending  # callback data
+         )
+      if installs:
+        self.pk_task.install_packages_async(installs,
+              self.cancellable,  # cancellable
+              self.on_driver_changes_progress,
+              (None, ),  # progress data
+              self.on_driver_changes_finish,  # GAsyncReadyCallback
+              False  # ready data
+       )
       self.button_driver_revert.setEnabled(False)
       self.button_driver_apply.setEnabled(False)
       #self.scrolled_window_drivers.set_sensitive(False)
-    except (NotAuthorizedError, TransactionFailed) as e:
+    except Exception as e:
       print("Warning: install transaction not completed successfully: {}".format(e))
 
   def on_driver_changes_revert(self, button_revert=None):
